import { ReactNode, RefObject } from 'react';
export interface UseFocusTrapProps {
    /**
     * Ref-container with dom-node for focus trap.
     */
    scopeRef: RefObject<HTMLElement>;
    /**
     * Enables focus trap.
     */
    enabled: boolean | undefined;
    /**
     * Returns focus in trigger after disabling trap.
     *
     * @default true
     */
    restoreFocus?: boolean;
    /**
     * Ref-container with dom-node uses for restore focus after disabling trap.
     */
    restoreFocusRef?: RefObject<HTMLElement>;
    /**
     * Sets focus in first focusable element inside trap.
     *
     * @default true
     */
    autoFocus?: boolean;
    /**
     * Children content
     *
     * @default undefined
     */
    children?: ReactNode;
}
/**
 * Creates focus trap for selected dom-node.
 *
 * @example
 * const Modal = (props) => {
 *   const { isVisible, children } = props
 *   const scopeRef = useRef(null)
 *
 *   useFocusTrap({ enabled: isVisible, scopeRef })
 *
 *   return (
 *     <div ref={scopeRef}>{children}</div>
 *   )
 * }
 */
export declare function useFocusTrap(props: UseFocusTrapProps): void;
