import { RefObject } from 'react';
export interface UseRestoreFocusProps {
    /**
     * Enables focus restore (saves last focused element).
     */
    enabled: boolean | undefined;
    /**
     * Ref-container with dom-node uses for restore focus after disabling.
     *
     * @default {document.activeElement}
     */
    restoreFocusRef?: RefObject<HTMLElement>;
}
/**
 * Restores focus in last focused element.
 *
 * @example
 * const Modal = (props) => {
 *   const { isVisible } = props
 *
 *   useRestoreFocus({ enabled: isVisible })
 *   ...
 * }
 */
export declare function useRestoreFocus(props: UseRestoreFocusProps): void;
