import { __rest } from "tslib";
import { focusElement } from '../dom-utils';
import { FocusNavigation } from './FocusNavigation';
export function createFocusManager(scopeRef) {
    var instance = null;
    function ensureNavigation(callback) {
        if (scopeRef.current && (instance === null || instance === void 0 ? void 0 : instance.scope) !== scopeRef.current) {
            instance = new FocusNavigation(scopeRef.current);
        }
        if (instance) {
            return callback(instance);
        }
        return null;
    }
    function focusFirst(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, tabbable = options.tabbable;
        var element = ensureNavigation(function (navigation) { return navigation.first(tabbable); });
        focusElement(element, preventScroll);
        return element;
    }
    function focusLast(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, tabbable = options.tabbable;
        var element = ensureNavigation(function (navigation) { return navigation.last(tabbable); });
        focusElement(element, preventScroll);
        return element;
    }
    function focusNext(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, opts = __rest(options, ["preventScroll"]);
        var element = ensureNavigation(function (navigation) { return navigation.next(opts); });
        focusElement(element, preventScroll);
        return element;
    }
    function focusPrevious(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, opts = __rest(options, ["preventScroll"]);
        var element = ensureNavigation(function (navigation) { return navigation.previous(opts); });
        focusElement(element, preventScroll);
        return element;
    }
    return {
        focusFirst: focusFirst,
        focusLast: focusLast,
        focusNext: focusNext,
        focusPrevious: focusPrevious,
    };
}
