import { useImperativeHandle } from 'react';
/**
 * Copies forwarded ref to mutable ref object.
 *
 * @example
 * const Component = forwardRef<HTMLDivElement>((_, forwardedRef) => {
 *   const ref = useRef<HTMLDivElement>(null)
 *   useForwardedRef(ref, forwardedRef)
 *
 *   return <div ref={ref} />
 * })
 */
export function useForwardedRef(ref, forwardedRef) {
    useImperativeHandle(forwardedRef, function () { return ref.current; });
}
