var PRIORITIES = ['assertive', 'polite'];
var LiveAnnouncer = /** @class */ (function () {
    function LiveAnnouncer() {
        this.liveRegions = {};
    }
    LiveAnnouncer.prototype.announce = function (message, priority) {
        var liveRegion = this.getLiveRegion(priority);
        var announceMessage = this.lastMessage === message ? "".concat(message, "\u00A0") : message;
        if (message) {
            this.lastMessage = announceMessage;
        }
        liveRegion.textContent = announceMessage;
    };
    LiveAnnouncer.prototype.dispose = function () {
        var _this = this;
        PRIORITIES.forEach(function (priority) {
            var liveRegion = _this.liveRegions[priority];
            if (liveRegion) {
                document.body.removeChild(liveRegion);
            }
        });
        this.liveRegions = {};
    };
    LiveAnnouncer.prototype.getLiveRegion = function (priority) {
        var liveRegion = this.liveRegions[priority];
        if (!liveRegion) {
            var node = document.createElement('div');
            node.setAttribute('aria-live', priority);
            node.setAttribute('aria-atomic', 'true');
            Object.assign(node.style, {
                border: 0,
                clip: 'rect(0 0 0 0)',
                clipPath: 'inset(50%)',
                height: 1,
                margin: '0 -1px -1px 0',
                overflow: 'hidden',
                padding: 0,
                position: 'absolute',
                width: 1,
                whiteSpace: 'nowrap',
            });
            this.liveRegions[priority] = node;
            document.body.appendChild(node);
            return node;
        }
        return liveRegion;
    };
    return LiveAnnouncer;
}());
var liveAnnouncer = null;
/**
 * Cause a message to be announced by screen readers.
 *
 * @param message The message that should be announced.
 * @param priority The priority with which screen reader should treat updates. Default: `polite`
 */
export function announce(message, priority) {
    if (priority === void 0) { priority = 'polite'; }
    if (!liveAnnouncer) {
        liveAnnouncer = new LiveAnnouncer();
    }
    liveAnnouncer.announce(message, priority);
}
export function disposeLiveAnnouncer() {
    if (liveAnnouncer) {
        liveAnnouncer.dispose();
        liveAnnouncer = null;
    }
}
