import { canUseDom } from '../dom-utils';
import { isIOS } from '../platform';
import * as StandartScrollLocker from './StandartScrollLocker';
import * as TouchScrollLocker from './TouchScrollLocker';
function ensureElement(element) {
    return element || document.body;
}
/**
 * Adds a lock on content scrolling on a DOM-element.
 *
 * @param container Link on DOM-element. Default: `document.body`
 */
export function lock(container) {
    if (!canUseDom) {
        return;
    }
    var element = ensureElement(container);
    StandartScrollLocker.lock(element);
    if (isIOS()) {
        TouchScrollLocker.lock(element);
    }
}
/**
 * Removes the lock on content scrolling on a DOM-element.
 *
 * @param container Link on DOM-element. Default: `document.body`
 */
export function unlock(container) {
    if (!canUseDom) {
        return;
    }
    var element = ensureElement(container);
    StandartScrollLocker.unlock(element);
    if (isIOS()) {
        TouchScrollLocker.unlock(element);
    }
}
