import { getScrollBarGap, isRootHTMLElement, setStyle } from './utils';
var randomKey = Math.random().toString(36).slice(2);
var SCROLL_LOCK_STATE_KEY = "__scrollLockState$".concat(randomKey);
function getLockState(node) {
    return node[SCROLL_LOCK_STATE_KEY];
}
function setLockState(node, state) {
    node[SCROLL_LOCK_STATE_KEY] = state;
}
function clearLockState(node) {
    delete node[SCROLL_LOCK_STATE_KEY];
}
function hasStaticVerticalScroll(node) {
    var overflowY = getComputedStyle(node).overflowY;
    return /scroll/.test(overflowY);
}
function getScrollbarSize(node) {
    var hasScrollbarRoot = isRootHTMLElement(node) && window.innerWidth - document.documentElement.clientWidth > 0;
    var hasScrollbarNode = node.scrollHeight > node.clientHeight;
    if (hasScrollbarRoot || hasScrollbarNode || hasStaticVerticalScroll(node)) {
        return getScrollBarGap();
    }
    return 0;
}
export function lock(container) {
    var state = getLockState(container);
    if (state) {
        state.count++;
        return;
    }
    var scrollBarSize = getScrollbarSize(container);
    var computedPaddingRight = parseInt(getComputedStyle(container).getPropertyValue('padding-right'), 10);
    var initialStyle = setStyle(container, {
        paddingRight: "".concat(computedPaddingRight + scrollBarSize, "px"),
        overflow: 'hidden',
        overflowX: 'hidden',
        overflowY: 'hidden',
    });
    setLockState(container, { initialStyle: initialStyle, count: 1 });
}
export function unlock(container) {
    var state = getLockState(container);
    if (!state) {
        return;
    }
    state.count--;
    if (state.count === 0) {
        setStyle(container, state.initialStyle);
        clearLockState(container);
    }
}
