import { __read } from "tslib";
import { useCallback, useRef, useState } from 'react';
/**
 * Manages state for single and multiple selection.
 *
 * @example
 * const Component = (props) => {
 *   const selectionState = useSelectionState({
 *      selectionMode: 'multiple',
 *      defaultSelectedKeys: ['a', 'b'],
 *      onSelectionChange: (keys) => {
 *         // ...
 *      },
 *   });
 * })
 */
export function useSelectionState(props) {
    var defaultSelectedKeys = props.defaultSelectedKeys, onSelectionChange = props.onSelectionChange, selectionMode = props.selectionMode, _a = props.disallowEmptySelection, disallowEmptySelection = _a === void 0 ? false : _a;
    var _b = __read(useState(function () { return new Set(defaultSelectedKeys); }), 2), selectedKeys = _b[0], setSelectedKeys = _b[1];
    var onSelectionChangeRef = useRef(onSelectionChange);
    var select = useCallback(function (key) {
        var _a;
        var isExists = selectedKeys.has(key);
        if (selectionMode === 'none' ||
            (isExists && disallowEmptySelection && selectedKeys.size === 1)) {
            return;
        }
        var keys = new Set(selectedKeys);
        if (isExists) {
            keys.delete(key);
        }
        else {
            if (selectionMode === 'single') {
                keys.clear();
            }
            keys.add(key);
        }
        setSelectedKeys(keys);
        (_a = onSelectionChangeRef.current) === null || _a === void 0 ? void 0 : _a.call(onSelectionChangeRef, keys);
    }, [selectedKeys, selectionMode, disallowEmptySelection]);
    var isSelected = useCallback(function (key) { return selectedKeys.has(key); }, [selectedKeys]);
    return {
        selectedKeys: selectedKeys,
        isSelected: isSelected,
        select: select,
    };
}
