import { Children } from 'react';
import { isSlotComponent, isSlotElement } from './utils';
var SlotMap = /** @class */ (function () {
    function SlotMap(children, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.children = null;
        var defaultSlot = options.defaultSlot;
        var defaultContent = [];
        this.slots = new Map();
        if (defaultSlot && !isSlotComponent(defaultSlot)) {
            throw new Error('Invalid default slot component. Should be a component created using "createSlot".');
        }
        Children.forEach(children, function (child) {
            if (isSlotElement(child)) {
                var type = child.type, props = child.props;
                if (_this.slots.has(type)) {
                    throw new Error("Duplicate slot elements with name \"".concat(type.__slotName, "\" found."));
                }
                _this.slots.set(type, {
                    name: type.__slotName,
                    props: props,
                    rendered: props.children,
                });
            }
            else if (child !== null && child !== undefined) {
                defaultContent.push(child);
            }
        });
        var defaultSlotItem = defaultSlot ? this.slots.get(defaultSlot) : undefined;
        var hasContent = defaultContent.length > 0;
        if (defaultSlotItem && hasContent) {
            throw new Error("Extraneous children found when component already has explicitly default slot with name \"".concat(defaultSlotItem.name, "\"."));
        }
        if (defaultSlotItem) {
            this.children = defaultSlotItem.rendered;
        }
        else if (hasContent) {
            this.children = defaultContent;
        }
    }
    SlotMap.prototype.get = function (slot) {
        if (!isSlotComponent(slot)) {
            throw new Error('Invalid slot component. Should be a component created using "createSlot".');
        }
        return this.slots.get(slot);
    };
    return SlotMap;
}());
export { SlotMap };
