import { addMonths, addYears, endOfMonth, endOfYear, isSameDay, isSameMonth, isSameYear, set, startOfDay, startOfMonth, startOfYear, subMonths, subYears, } from '../../../internal/date';
import { clamp } from '../../../libs/utils';
import { durationInDayViews, getDayViewData, moveInDayView } from './day-view';
import { durationInMonthViews, getMonthViewData, moveInMonthView } from './month-view';
import { addYearViews, durationInYearViews, endOfYearView, getYearViewData, moveInYearView, startOfYearView, subYearViews, } from './year-view';
export { isEqual as isEqualDate } from '../../../internal/date';
export function createRange(start, end) {
    if (!start || !end) {
        return { start: start !== null && start !== void 0 ? start : end, end: null };
    }
    if (end < start) {
        return { start: end, end: start };
    }
    return { start: start, end: end };
}
export function startOfView(view, value) {
    var utils = {
        day: startOfMonth,
        month: startOfYear,
        year: startOfYearView,
    };
    return utils[view](value);
}
export function endOfView(view, date) {
    var utils = {
        day: endOfMonth,
        month: endOfYear,
        year: endOfYearView,
    };
    return utils[view](date);
}
export function startOfCell(view, date) {
    var utils = {
        day: startOfDay,
        month: startOfMonth,
        year: startOfYear,
    };
    return utils[view](date);
}
export function durationInViews(view, start, end) {
    var utils = {
        day: durationInDayViews,
        month: durationInMonthViews,
        year: durationInYearViews,
    };
    return utils[view](start, end);
}
export function addViews(view, date, amount) {
    var utils = {
        day: addMonths,
        month: addYears,
        year: addYearViews,
    };
    return utils[view](date, amount);
}
export function subViews(view, date, amount) {
    var utils = {
        day: subMonths,
        month: subYears,
        year: subYearViews,
    };
    return utils[view](date, amount);
}
export function nearest(view, candidate, min, total) {
    var distanceToStart = durationInViews(view, candidate, min);
    var distanceToEnd = distanceToStart + total - 1;
    if (distanceToStart > 0 || distanceToEnd < 0) {
        var amount = Math.abs(distanceToStart) < Math.abs(distanceToEnd) ? distanceToStart : distanceToEnd;
        return addViews(view, candidate, amount);
    }
    return candidate;
}
export function isSameView(view, leftDate, rightDate) {
    return durationInViews(view, leftDate, rightDate) === 0;
}
export function isInRange(view, candidate, range) {
    if (!range.start || !range.end) {
        return false;
    }
    var value = startOfCell(view, candidate);
    return value >= startOfCell(view, range.start) && value <= startOfCell(view, range.end);
}
export function isSameCell(view, leftDate, rightDate) {
    var utils = {
        day: isSameDay,
        month: isSameMonth,
        year: isSameYear,
    };
    return utils[view](leftDate, rightDate);
}
export function clampDate(date, min, max) {
    var timestamp = clamp(date.getTime(), min.getTime(), max.getTime());
    return startOfDay(timestamp);
}
export function getViewData(view, viewDate, options) {
    var utils = {
        day: getDayViewData,
        month: getMonthViewData,
        year: getYearViewData,
    };
    return utils[view](viewDate, options);
}
export function move(view, date, action, options) {
    var utils = {
        day: moveInDayView,
        month: moveInMonthView,
        year: moveInYearView,
    };
    return utils[view](date, action, options);
}
export function closestViewDate(view, start, total, target) {
    var maxIndex = total - 1;
    var duration = durationInViews(view, start, target);
    var offset = Math.min(duration, 0) + Math.max(duration, maxIndex) - maxIndex;
    if (offset !== 0) {
        return addViews(view, start, offset);
    }
    return start;
}
// TODO: rename
export function getViewDate(view, baseDate, focusedDate) {
    var day = focusedDate.getDate();
    var month = focusedDate.getMonth();
    var utils = {
        day: function (v) { return v; },
        month: function (v) { return set(v, { date: day }); },
        year: function (v) { return set(v, { date: day, month: month }); },
    };
    return utils[view](baseDate);
}
export function getViews(view, baseDate, count, options) {
    var result = [];
    for (var index = 0; index < count; index++) {
        var viewDate = addViews(view, baseDate, index);
        var start = startOfView(view, viewDate);
        var end = endOfView(view, viewDate);
        var viewRange = createRange(start, end);
        result.push({
            viewDate: viewDate,
            viewRange: viewRange,
            data: getViewData(view, viewDate, options),
        });
    }
    return result;
}
