import { addDays, addMonths, addWeeks, addYears, endOfWeek, getDaysInMonth, lastDayOfMonth, startOfMonth, startOfWeek, subDays, subMonths, subWeeks, subYears, } from '../../../internal/date';
import { CalendarNavigationAction } from '../types';
export function durationInDayViews(start, end) {
    return (end.getFullYear() - start.getFullYear()) * 12 + (end.getMonth() - start.getMonth());
}
export function getDayViewData(viewDate, options) {
    if (options === void 0) { options = {}; }
    var _a = options.firstDayOfWeek, firstDayOfWeek = _a === void 0 ? 0 : _a;
    var data = [];
    var month = viewDate.getMonth();
    var year = viewDate.getFullYear();
    var monthStartsAt = (startOfMonth(viewDate).getDay() - firstDayOfWeek) % 7;
    if (monthStartsAt < 0) {
        monthStartsAt += 7;
    }
    var daysInMonth = getDaysInMonth(viewDate);
    var weeksInMonth = Math.ceil((monthStartsAt + daysInMonth) / 7);
    for (var weekIndex = 0; weekIndex < weeksInMonth; weekIndex++) {
        data.push([]);
        for (var dayIndex = 0; dayIndex < 7; dayIndex++) {
            var day = weekIndex * 7 + dayIndex - monthStartsAt + 1;
            var value = new Date(year, month, day);
            data[weekIndex].push(value);
        }
    }
    return data;
}
export function moveInDayView(date, action, options) {
    if (options === void 0) { options = {}; }
    var firstDayOfWeek = options.firstDayOfWeek;
    switch (action) {
        case CalendarNavigationAction.NextCell:
            return addDays(date, 1);
        case CalendarNavigationAction.PrevCell:
            return subDays(date, 1);
        case CalendarNavigationAction.LowerCell:
            return addWeeks(date, 1);
        case CalendarNavigationAction.UpperCell:
            return subWeeks(date, 1);
        case CalendarNavigationAction.FirstCell:
            return startOfMonth(date);
        case CalendarNavigationAction.LastCell:
            return lastDayOfMonth(date);
        case CalendarNavigationAction.StartCell:
            return startOfWeek(date, { weekStartsOn: firstDayOfWeek });
        case CalendarNavigationAction.EndCell:
            return endOfWeek(date, { weekStartsOn: firstDayOfWeek });
        case CalendarNavigationAction.PrevView:
            return subMonths(date, 1);
        case CalendarNavigationAction.NextView:
            return addMonths(date, 1);
        case CalendarNavigationAction.PrevExtraView:
            return subYears(date, 1);
        case CalendarNavigationAction.NextExtraView:
            return addYears(date, 1);
        default:
            return date;
    }
}
