import { addYears, endOfYear, setYear, startOfYear, subYears } from '../../../internal/date';
import { CalendarNavigationAction } from '../types';
var YEAR_VIEW_ROW_COUNT = 6;
var YEAR_VIEW_COL_COUNT = 4;
var YEAR_VIEW_TOTAL_COUNT = YEAR_VIEW_ROW_COUNT * YEAR_VIEW_COL_COUNT;
function getStartYearOfView(date) {
    return Math.floor(date.getFullYear() / YEAR_VIEW_TOTAL_COUNT) * YEAR_VIEW_TOTAL_COUNT;
}
function getEndYearOfView(date) {
    return getStartYearOfView(date) + YEAR_VIEW_TOTAL_COUNT - 1;
}
export function startOfYearView(date) {
    var year = getStartYearOfView(date);
    return startOfYear(setYear(date, year));
}
export function endOfYearView(date) {
    var year = getEndYearOfView(date);
    return endOfYear(setYear(date, year));
}
export function durationInYearViews(start, end) {
    var startYear = getStartYearOfView(start);
    var endYear = getStartYearOfView(end);
    return (endYear - startYear) / YEAR_VIEW_TOTAL_COUNT;
}
export function addYearViews(date, amount) {
    return addYears(date, amount * YEAR_VIEW_TOTAL_COUNT);
}
export function subYearViews(date, amount) {
    return subYears(date, amount * YEAR_VIEW_TOTAL_COUNT);
}
export function getYearViewData(viewDate) {
    var data = [];
    for (var row = 0, index = 0; row < YEAR_VIEW_ROW_COUNT; row++) {
        data.push([]);
        for (var col = 0; col < YEAR_VIEW_COL_COUNT; col++) {
            data[row].push(addYears(viewDate, index++));
        }
    }
    return data;
}
export function moveInYearView(date, action) {
    switch (action) {
        case CalendarNavigationAction.NextCell:
            return addYears(date, 1);
        case CalendarNavigationAction.PrevCell:
            return subYears(date, 1);
        case CalendarNavigationAction.LowerCell:
            return addYears(date, YEAR_VIEW_COL_COUNT);
        case CalendarNavigationAction.UpperCell:
            return subYears(date, YEAR_VIEW_COL_COUNT);
        case CalendarNavigationAction.StartCell:
        case CalendarNavigationAction.FirstCell:
            return setYear(date, getStartYearOfView(date));
        case CalendarNavigationAction.EndCell:
        case CalendarNavigationAction.LastCell:
            return setYear(date, getEndYearOfView(date));
        case CalendarNavigationAction.PrevView:
            return subYearViews(date, 1);
        case CalendarNavigationAction.NextView:
            return addYearViews(date, 1);
        default:
            return date;
    }
}
