import { __assign, __read } from "tslib";
import { useCallback, useState } from 'react';
import { usePress } from '../../interactions/press';
import { useFocusManager } from '../../libs/focus';
import { useLocale } from '../../libs/i18n';
import { mergeProps } from '../../libs/merge-props';
import { useSpinButton } from '../spinbutton';
import { getAriaValueAttributes, parseSegmentValue } from './utils';
export function useDateTimeFieldSegment(props, state) {
    var segment = props.segment;
    var type = segment.type, value = segment.value, max = segment.max, text = segment.text, isValid = segment.isValid, isDisabled = segment.isDisabled;
    var resolvedOptions = state.resolvedOptions, increment = state.increment, decrement = state.decrement, extraIncrement = state.extraIncrement, extraDecrement = state.extraDecrement, incrementToMax = state.incrementToMax, decrementToMin = state.decrementToMin, setSegmentValue = state.setSegmentValue;
    var isRTL = useLocale().isRTL;
    var _a = useFocusManager(), focusNext = _a.focusNext, focusPrevious = _a.focusPrevious;
    var _b = __read(useState(''), 2), enteredKeys = _b[0], setEnteredKeys = _b[1];
    var ariaValueAttrs = getAriaValueAttributes(segment, resolvedOptions);
    var spinButtonProps = useSpinButton(__assign(__assign({}, ariaValueAttrs), { textValue: text, disabled: isDisabled, onIncrement: useCallback(function () { return increment(type); }, [increment, type]), onDecrement: useCallback(function () { return decrement(type); }, [decrement, type]), onExtraIncrement: useCallback(function () { return extraIncrement(type); }, [extraIncrement, type]), onExtraDecrement: useCallback(function () { return extraDecrement(type); }, [extraDecrement, type]), onIncrementToMax: useCallback(function () { return incrementToMax(type); }, [incrementToMax, type]), onDecrementToMin: useCallback(function () { return decrementToMin(type); }, [decrementToMin, type]) })).spinButtonProps;
    var pressProps = usePress({
        onPressStart: function (event) {
            if (event.source === 'mouse') {
                event.currentTarget.focus();
            }
        },
    }).pressProps;
    var setValue = useCallback(function (v) {
        setSegmentValue(type, v);
        if (v === null || v * 10 >= max) {
            setEnteredKeys('');
        }
    }, [max, setSegmentValue, type]);
    var onKeyDown = useCallback(function (event) {
        // TODO: check the use of special keys for entering digits in different locales (ex. ar-AE)
        if (['Tab', 'Shift', 'Alt', 'Meta'].includes(event.key) || event.ctrlKey || event.metaKey) {
            return;
        }
        event.preventDefault();
        var focusOptions = { from: event.currentTarget, tabbable: true };
        switch (event.key) {
            case 'ArrowLeft': {
                var focusElement = isRTL ? focusNext : focusPrevious;
                focusElement(focusOptions);
                break;
            }
            case 'ArrowRight': {
                var focusElement = isRTL ? focusPrevious : focusNext;
                focusElement(focusOptions);
                break;
            }
            case 'Backspace': {
                if (isDisabled || value === null) {
                    focusPrevious(focusOptions);
                }
                else {
                    setValue(null);
                }
                break;
            }
            default:
                var newValue = enteredKeys + event.key;
                var numberValue = parseSegmentValue(type, newValue);
                // TODO: check readOnly
                if (isDisabled || numberValue === null) {
                    return;
                }
                event.stopPropagation();
                setValue(numberValue);
                var maxLen = max.toString().length;
                // if the number with next digit capacity is greater than the maximum,
                // move the focus to the next segment
                if (newValue.length >= maxLen || numberValue * 10 >= max) {
                    focusNext(focusOptions);
                }
                else {
                    setEnteredKeys(newValue);
                }
        }
    }, [enteredKeys, focusNext, focusPrevious, isDisabled, isRTL, max, setValue, type, value]);
    var segmentProps = {
        // TODO: set aria props
        // 'aria-label': '',
        // 'aria-labelledby': '',
        // TODO: may be move to useDateTimeField?
        'aria-invalid': !isValid,
        tabIndex: isDisabled ? undefined : 0,
        onKeyDown: onKeyDown,
        onFocus: useCallback(function () {
            setEnteredKeys('');
        }, []),
    };
    if (!isDisabled) {
        segmentProps.inputMode = 'numeric';
        // TODO: Voice over incorrect work with contentEditable
        segmentProps.contentEditable = true;
        segmentProps.suppressContentEditableWarning = true;
    }
    return {
        segmentProps: mergeProps(spinButtonProps, pressProps, segmentProps),
    };
}
