import { useFocusable } from '../../interactions/focusable';
import { usePress } from '../../interactions/press';
import { mergeProps } from '../../libs/merge-props';
import { isFirefox } from '../../libs/platform';
import { useRadioGroupContext } from './useRadioGroupContext';
export function useRadio(props, inputRef) {
    var _a;
    var propChecked = props.checked, defaultChecked = props.defaultChecked, disabled = props.disabled, name = props.name, onChange = props.onChange, propReadOnly = props.readOnly, state = props.state, value = props.value;
    var focusableProps = useFocusable(props, inputRef).focusableProps;
    var _b = usePress(props), isPressed = _b.isPressed, pressProps = _b.pressProps;
    var radioGroupContext = useRadioGroupContext();
    var checked = radioGroupContext
        ? radioGroupContext.selectedValue === props.value
        : propChecked !== null && propChecked !== void 0 ? propChecked : defaultChecked;
    var readOnly = (radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.isReadOnly) || propReadOnly;
    if (radioGroupContext) {
        if (process.env.NODE_ENV !== 'production') {
            if (props.checked || props.defaultChecked) {
                console.warn('Using checked/defaultChecked prop with RadioGroupContext will have no effect');
            }
            if (props.onChange) {
                console.warn('Using onChange prop with RadioGroupContext will have no effect');
            }
            if (props.name) {
                console.warn('Using name prop with RadioGroupContext will have no effect');
            }
        }
    }
    var handleOnChange = function (event) {
        var _a;
        if (radioGroupContext) {
            (_a = radioGroupContext.setSelectedValue) === null || _a === void 0 ? void 0 : _a.call(radioGroupContext, event);
        }
        else {
            onChange === null || onChange === void 0 ? void 0 : onChange(event);
        }
    };
    return {
        isPressed: isPressed,
        rootProps: pressProps,
        inputProps: mergeProps(focusableProps, {
            type: 'radio',
            onChange: readOnly ? undefined : handleOnChange,
            'aria-invalid': state === 'invalid' || undefined,
            'aria-checked': checked || props.defaultChecked,
            // Use "aria-readonly" because "readOnly" available only for text fields,
            // see https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly.
            'aria-readonly': readOnly || undefined,
            // https://bugzilla.mozilla.org/show_bug.cgi?id=654072
            autoComplete: isFirefox() ? 'off' : undefined,
            checked: checked,
            name: (_a = radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.name) !== null && _a !== void 0 ? _a : name,
            disabled: (radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.isDisabled) || disabled,
            value: value,
        }),
    };
}
