import { useIsomorphicLayoutEffect } from '../../libs/isomorphic-layout-effect';
import { clamp } from '../../libs/utils';
/**
 * Enable auto resize textarea when text changed.
 *
 * @param props - Props for the auto resize.
 * @param inputRef - Ref to the textarea html element.
 */
export function useAutoResize(props, inputRef) {
    var value = props.value, enabled = props.enabled;
    var minRows = Math.max(props.minRows || 1, 1);
    var maxRows = clamp(props.maxRows || Infinity, minRows, Infinity);
    useIsomorphicLayoutEffect(function () {
        var input = inputRef.current;
        if (input) {
            input.rows = minRows;
            if (!enabled) {
                return;
            }
            for (var rows = minRows + 1; rows <= maxRows; rows++) {
                if (input.scrollHeight <= input.clientHeight) {
                    break;
                }
                input.rows = rows;
            }
        }
    }, [inputRef, value, enabled]);
}
