"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useOutsideClick = void 0;
var react_1 = require("react");
function isEssentionalClick(refs, target) {
    return refs.some(function (ref) {
        return ref.current instanceof HTMLElement && ref.current.contains(target);
    });
}
function useOutsideClick(props) {
    var onAction = props.onAction, refs = props.refs, triggerStrategy = props.triggerStrategy, disabled = props.disabled;
    var partialPropsRef = (0, react_1.useRef)({
        onAction: onAction,
        refs: refs,
    });
    partialPropsRef.current = { onAction: onAction, refs: refs };
    (0, react_1.useEffect)(function () {
        var currentTarget = null;
        var targetCapturer = function (event) {
            currentTarget = event.target;
        };
        var eventType = triggerStrategy === 'pressup' ? 'click' : 'pointerdown';
        var listener = function (event) {
            if (event.button > 0 || disabled) {
                return;
            }
            if (triggerStrategy === 'pressup') {
                var isSameTarget = currentTarget === event.target;
                currentTarget = null;
                if (!isSameTarget) {
                    return;
                }
            }
            if (!isEssentionalClick(partialPropsRef.current.refs, event.target)) {
                partialPropsRef.current.onAction(event);
            }
        };
        document.addEventListener(eventType, listener);
        document.addEventListener('pointerdown', targetCapturer);
        return function () {
            document.removeEventListener(eventType, listener);
            document.removeEventListener('pointerdown', targetCapturer);
        };
    }, [triggerStrategy, disabled]);
}
exports.useOutsideClick = useOutsideClick;
