"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePress = void 0;
var tslib_1 = require("tslib");
var react_1 = require("react");
var useListeners_1 = require("../../internal/useListeners");
var dom_utils_1 = require("../../libs/dom-utils");
var dom_utils_2 = require("../../libs/dom-utils");
var create_press_event_1 = require("./utils/create-press-event");
var detect_overlap_1 = require("./utils/detect-overlap");
var keyboard_event_1 = require("./utils/keyboard-event");
var text_selection_1 = require("./utils/text-selection");
var touch_event_1 = require("./utils/touch-event");
function usePress(props) {
    var preventFocusOnPress = props.preventFocusOnPress;
    var _a = (0, useListeners_1.useListeners)(), addListener = _a.addListener, removeAllListeners = _a.removeAllListeners;
    var _b = tslib_1.__read((0, react_1.useState)(false), 2), isPressed = _b[0], setPressed = _b[1];
    var cacheRef = (0, react_1.useRef)({
        currentPointerId: null,
        // Expect that the currentTarget is always exists
        currentPointerTarget: null,
        isPressed: false,
        isPressStarted: false,
    });
    var propsRef = (0, react_1.useRef)({});
    // Use ref as cache for reuse props inside memo hook.
    propsRef.current = {
        disabled: props.disabled,
        onPressStart: props.onPressStart,
        onPressUp: props.onPressUp,
        onPressEnd: props.onPressEnd,
        onPress: props.onPress,
    };
    var pressProps = (0, react_1.useMemo)(function () {
        var cache = cacheRef.current;
        var props = {
            onKeyDown: function (event) {
                if (event.key === 'Tab' && cache.isPressStarted) {
                    event.preventDefault();
                }
                if ((0, keyboard_event_1.isValidKeyboardEvent)(event.nativeEvent)) {
                    // Use preventDefault for all elements except checkbox and radiobox inputs,
                    // because input should trigger onChange after keydown.
                    if (!(0, keyboard_event_1.isCheckableInput)(event.target)) {
                        // Use preventDefault for stop document scroll for interactive elements.
                        event.preventDefault();
                    }
                    event.stopPropagation();
                    if (!cache.isPressed && !event.repeat) {
                        triggerPressStart((0, create_press_event_1.createPressEvent)(event, event.currentTarget, 'keyboard'));
                    }
                }
            },
            // TODO: Register as global listener after keydown.
            onKeyUp: function (event) {
                if ((0, keyboard_event_1.isValidKeyboardEvent)(event.nativeEvent) && !event.repeat) {
                    triggerPressUp((0, create_press_event_1.createPressEvent)(event, event.currentTarget, 'keyboard'));
                    triggerPressEnd((0, create_press_event_1.createPressEvent)(event, event.currentTarget, 'keyboard'));
                }
            },
        };
        var triggerPressStart = function (event) {
            var _a = propsRef.current, disabled = _a.disabled, onPressStart = _a.onPressStart;
            if (disabled || cache.isPressStarted) {
                return;
            }
            setPressed(true);
            cache.isPressStarted = true;
            onPressStart === null || onPressStart === void 0 ? void 0 : onPressStart(tslib_1.__assign(tslib_1.__assign({}, event), { type: 'pressstart' }));
        };
        var triggerPressUp = function (event) {
            var _a = propsRef.current, disabled = _a.disabled, onPressUp = _a.onPressUp;
            if (disabled) {
                return;
            }
            onPressUp === null || onPressUp === void 0 ? void 0 : onPressUp(tslib_1.__assign(tslib_1.__assign({}, event), { type: 'pressup' }));
        };
        var triggerPressEnd = function (event, triggerOnPress) {
            if (triggerOnPress === void 0) { triggerOnPress = true; }
            var _a = propsRef.current, onPress = _a.onPress, onPressEnd = _a.onPressEnd;
            if (!cache.isPressStarted) {
                return;
            }
            setPressed(false);
            cache.isPressStarted = false;
            onPressEnd === null || onPressEnd === void 0 ? void 0 : onPressEnd(tslib_1.__assign(tslib_1.__assign({}, event), { type: 'pressend' }));
            if (triggerOnPress) {
                onPress === null || onPress === void 0 ? void 0 : onPress(tslib_1.__assign(tslib_1.__assign({}, event), { type: 'press' }));
            }
        };
        var attach = function (target, id) {
            cache.currentPointerTarget = target;
            cache.currentPointerId = id;
            cache.isPressed = true;
            (0, text_selection_1.disableTextSelection)();
            setPressed(true);
        };
        var detach = function () {
            if (cache.isPressed) {
                cache.isPressed = false;
                (0, text_selection_1.restoreTextSelection)();
                setPressed(false);
                removeAllListeners();
            }
        };
        if (typeof PointerEvent !== 'undefined') {
            var onPointerMove_1 = function (event) {
                var pointerType = event.pointerType;
                if ((0, detect_overlap_1.isTargetContainsPoint)(cache.currentPointerTarget, event)) {
                    triggerPressStart((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, pointerType));
                }
                else {
                    triggerPressEnd((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, pointerType), false);
                }
            };
            var onPointerUp_1 = function (event) {
                var _a;
                // Dispose press only if down and up pointer ids are matches.
                if (event.pointerId === cache.currentPointerId) {
                    detach();
                    if ((0, detect_overlap_1.isTargetContainsPoint)(cache.currentPointerTarget, event)) {
                        var pointerType = event.pointerType;
                        triggerPressUp((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, pointerType));
                        triggerPressEnd((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, pointerType));
                        // Preventing extraneous click on the wrong element when tapping on overlay elements
                        (_a = event.target) === null || _a === void 0 ? void 0 : _a.addEventListener('touchend', function (e) {
                            var target = e.target;
                            if (!document.body.contains(target) || (0, dom_utils_1.isHidden)(target)) {
                                e.preventDefault();
                            }
                        }, {
                            once: true,
                            passive: false,
                        });
                    }
                }
            };
            // Cancel event can be fired while scroll.
            var onPointerCancel_1 = function (event) {
                if (cache.isPressed) {
                    var pointerType = event.pointerType;
                    triggerPressEnd((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, pointerType), false);
                }
                detach();
            };
            // fixes iOS VoiceOver bug when unexpected mouseDown event is fired and propagated to body (https://github.com/use-platform/use-platform/issues/205)
            props.onMouseDown = function (event) {
                event.stopPropagation();
            };
            props.onPointerDown = function (event) {
                var disabled = propsRef.current.disabled;
                // Handle only left clicks.
                if (event.button !== 0) {
                    return;
                }
                event.preventDefault();
                event.stopPropagation();
                if (!cache.isPressed && !disabled) {
                    if (!preventFocusOnPress) {
                        (0, dom_utils_2.focusWithoutScrolling)(event.currentTarget);
                    }
                    attach(event.currentTarget, event.pointerId);
                    triggerPressStart((0, create_press_event_1.createPressEvent)(event, event.currentTarget, event.pointerType));
                    addListener(document, 'pointermove', onPointerMove_1, false);
                    addListener(document, 'pointerup', onPointerUp_1, false);
                    addListener(document, 'pointercancel', onPointerCancel_1, false);
                }
            };
        }
        else {
            var onTouchMove_1 = function (event) {
                var touch = (0, touch_event_1.getTouchById)(event, cache.currentPointerId);
                if (touch) {
                    if ((0, detect_overlap_1.isTargetContainsPoint)(cache.currentPointerTarget, touch)) {
                        triggerPressStart((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, 'touch'));
                    }
                    else {
                        triggerPressEnd((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, 'touch'), false);
                    }
                }
            };
            var onTouchEnd_1 = function (event) {
                var touch = (0, touch_event_1.getTouchById)(event, cache.currentPointerId);
                // Dispose press only if down and up pointer ids are matches.
                if ((touch === null || touch === void 0 ? void 0 : touch.identifier) === cache.currentPointerId) {
                    detach();
                    if ((0, detect_overlap_1.isTargetContainsPoint)(cache.currentPointerTarget, touch)) {
                        triggerPressUp((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, 'touch'));
                        triggerPressEnd((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, 'touch'));
                    }
                }
            };
            // Cancel event can be fired while scroll.
            var onTouchCancel_1 = function (event) {
                if (cache.isPressed) {
                    triggerPressEnd((0, create_press_event_1.createPressEvent)(event, cache.currentPointerTarget, 'touch'), false);
                }
                detach();
            };
            props.onTouchStart = function (event) {
                var disabled = propsRef.current.disabled;
                event.preventDefault();
                event.stopPropagation();
                var touch = (0, touch_event_1.getTouchFromEvent)(event.nativeEvent);
                if (touch && !cache.isPressed && !disabled) {
                    if (!preventFocusOnPress) {
                        (0, dom_utils_2.focusWithoutScrolling)(event.currentTarget);
                    }
                    attach(event.currentTarget, touch.identifier);
                    triggerPressStart((0, create_press_event_1.createPressEvent)(event, event.currentTarget, 'touch'));
                    addListener(document, 'touchmove', onTouchMove_1, false);
                    addListener(document, 'touchend', onTouchEnd_1, false);
                    addListener(document, 'touchcancel', onTouchCancel_1, false);
                }
            };
        }
        return props;
    }, [addListener, preventFocusOnPress, removeAllListeners]);
    (0, react_1.useEffect)(function () {
        return text_selection_1.restoreTextSelection;
    }, []);
    return {
        isPressed: isPressed,
        pressProps: pressProps,
    };
}
exports.usePress = usePress;
