"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTargetContainsPoint = void 0;
function isTargetContainsPoint(target, event) {
    if (!target) {
        return false;
    }
    // Use native API for detect overlap,
    // but jsdom not implement this api and we use fallback.
    if (document.elementFromPoint) {
        // Calculate pointer target because event.target returns for ios always first target.
        var element = document.elementFromPoint(event.clientX, event.clientY);
        return target.contains(element);
    }
    var rect = target.getBoundingClientRect();
    var pointRect = getPointClientRect(event);
    // Check if they cannot overlap on x axis.
    if (rect.left > pointRect.right || pointRect.left > rect.right) {
        return false;
    }
    // Check if they cannot overlap on y axis.
    if (rect.top > pointRect.bottom || pointRect.top > rect.bottom) {
        return false;
    }
    return true;
}
exports.isTargetContainsPoint = isTargetContainsPoint;
function getPointClientRect(point) {
    var offsetX = point.width / 2 || point.radiusX || 0;
    var offsetY = point.height / 2 || point.radiusY || 0;
    return {
        top: point.clientY - offsetY,
        right: point.clientX + offsetX,
        bottom: point.clientY + offsetY,
        left: point.clientX - offsetX,
    };
}
