import { RefObject } from 'react';
export interface UsePreventScrollOptions {
    /**
     * Enables/disables scrolling for an element.
     */
    enabled: boolean | undefined;
    /**
     * Reference to the DOM element whose scrolling should be disabled.
     *
     * @default document.body
     */
    containerRef?: RefObject<HTMLElement>;
}
/**
 * Raact hook that prevents content from scrolling on an element.
 *
 * @example
 * const Modal: FC<ModalProps> = (props) => {
 *   const { visible } = props;
 *   const containerRef = useRef(document.documentElement);
 *
 *   usePreventScroll({ enabled: visible, containerRef });
 *
 *   ...
 * }
 */
export declare function usePreventScroll(options: UsePreventScrollOptions): void;
