"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFocusTrap = exports.FocusTrap = void 0;
var tslib_1 = require("tslib");
var FocusTrapManager_1 = require("./FocusTrapManager");
var utils_1 = require("./utils");
var FocusTrap = /** @class */ (function () {
    function FocusTrap(manager, scope, options) {
        if (options === void 0) { options = {}; }
        this.options = { autoFocus: true };
        this.active = false;
        this.cleanupFns = [];
        this.manager = manager;
        this.scope = scope;
        this.setOptions(options);
    }
    /**
     * Returns trapped html element.
     */
    FocusTrap.prototype.getScope = function () {
        return this.scope;
    };
    /**
     * Activates focus trap.
     */
    FocusTrap.prototype.activate = function () {
        if (this.active) {
            return;
        }
        this.active = true;
        this.attachGuards();
        this.activateInManager();
    };
    /**
     * Deactivates focus trap.
     */
    FocusTrap.prototype.deactivate = function () {
        if (!this.active) {
            return;
        }
        this.active = false;
        this.cleanupFns.forEach(function (fn) { return fn(); });
        this.cleanupFns = [];
    };
    /**
     * Returns focus trap options.
     */
    FocusTrap.prototype.getOptions = function () {
        return this.options;
    };
    /**
     * Sets focus trap options.
     */
    FocusTrap.prototype.setOptions = function (options) {
        this.options = tslib_1.__assign(tslib_1.__assign({}, this.options), options);
    };
    FocusTrap.prototype.attachGuards = function () {
        var beforeElement = (0, utils_1.createGuard)(1);
        var afterElement = (0, utils_1.createGuard)(0);
        this.scope.insertAdjacentElement('beforebegin', beforeElement);
        this.scope.insertAdjacentElement('afterend', afterElement);
        this.cleanupFns.push(function () {
            (0, utils_1.removeElementFromDocument)(beforeElement);
            (0, utils_1.removeElementFromDocument)(afterElement);
        });
    };
    FocusTrap.prototype.activateInManager = function () {
        var _this = this;
        this.manager.activate(this);
        this.cleanupFns.push(function () {
            _this.manager.deactivate(_this);
        });
    };
    return FocusTrap;
}());
exports.FocusTrap = FocusTrap;
/**
 * Creates focus trap for selected dom-node.
 *
 * @example
 * createFocusTrap(scopeDomNode, { autoFocus: true })
 */
function createFocusTrap(scope, options) {
    if (options === void 0) { options = {}; }
    var manager = FocusTrapManager_1.FocusTrapManager.getInstance();
    return new FocusTrap(manager, scope, options);
}
exports.createFocusTrap = createFocusTrap;
