"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFocusTrap = void 0;
var react_1 = require("react");
var isomorphic_layout_effect_1 = require("../isomorphic-layout-effect");
var FocusTrap_1 = require("./FocusTrap");
var useRestoreFocus_1 = require("./useRestoreFocus");
/**
 * Creates focus trap for selected dom-node.
 *
 * @example
 * const Modal = (props) => {
 *   const { isVisible, children } = props
 *   const scopeRef = useRef(null)
 *
 *   useFocusTrap({ enabled: isVisible, scopeRef })
 *
 *   return (
 *     <div ref={scopeRef}>{children}</div>
 *   )
 * }
 */
function useFocusTrap(props) {
    var scopeRef = props.scopeRef, enabled = props.enabled, restoreFocusRef = props.restoreFocusRef, _a = props.restoreFocus, restoreFocus = _a === void 0 ? true : _a, _b = props.autoFocus, autoFocus = _b === void 0 ? true : _b;
    var instance = (0, react_1.useRef)(null);
    var ref = (0, react_1.useRef)(null);
    var trapOptions = (0, react_1.useMemo)(function () {
        return {
            autoFocus: autoFocus,
        };
    }, [autoFocus]);
    (0, react_1.useEffect)(function () {
        if (instance.current) {
            instance.current.setOptions(trapOptions);
        }
    }, [trapOptions]);
    (0, isomorphic_layout_effect_1.useIsomorphicLayoutEffect)(function () {
        if (ref.current === scopeRef.current) {
            return;
        }
        ref.current = scopeRef.current;
        if (instance.current) {
            instance.current.deactivate();
            instance.current = null;
        }
        if (ref.current) {
            instance.current = (0, FocusTrap_1.createFocusTrap)(ref.current, trapOptions);
            if (enabled) {
                instance.current.activate();
            }
        }
    });
    (0, isomorphic_layout_effect_1.useIsomorphicLayoutEffect)(function () {
        if (!enabled || !instance.current) {
            return;
        }
        instance.current.activate();
        return function () {
            var _a;
            (_a = instance.current) === null || _a === void 0 ? void 0 : _a.deactivate();
        };
    }, [enabled]);
    (0, useRestoreFocus_1.useRestoreFocus)({ enabled: enabled && restoreFocus, restoreFocusRef: restoreFocusRef });
}
exports.useFocusTrap = useFocusTrap;
