"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useRestoreFocus = void 0;
var react_1 = require("react");
var dom_utils_1 = require("../dom-utils");
/**
 * Restores focus in last focused element.
 *
 * @example
 * const Modal = (props) => {
 *   const { isVisible } = props
 *
 *   useRestoreFocus({ enabled: isVisible })
 *   ...
 * }
 */
function useRestoreFocus(props) {
    var enabled = props.enabled, restoreFocusRef = props.restoreFocusRef;
    var initialActiveElement = (0, react_1.useRef)(null);
    var ref = (0, react_1.useRef)(null);
    if (enabled && !initialActiveElement.current) {
        // For elements with autoFocus React sets focus programmatically,
        // because important catch active element at render phase.
        initialActiveElement.current = dom_utils_1.canUseDom ? document.activeElement : null;
    }
    (0, react_1.useEffect)(function () {
        ref.current = restoreFocusRef ? restoreFocusRef.current : null;
    });
    (0, react_1.useEffect)(function () {
        if (!enabled) {
            return;
        }
        return function () {
            var element = ref.current || initialActiveElement.current;
            initialActiveElement.current = null;
            ref.current = null;
            if (element &&
                element.focus &&
                document.body.contains(element) &&
                element !== document.activeElement) {
                element.focus();
            }
        };
    }, [enabled]);
}
exports.useRestoreFocus = useRestoreFocus;
