"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNextTabbableIndex = exports.focusFirstIn = exports.removeElementFromDocument = exports.isGuard = exports.createGuard = exports.isFocusFree = exports.isElementInScope = exports.getActiveElement = void 0;
var tslib_1 = require("tslib");
var dom_utils_1 = require("../../dom-utils");
var tabbable_1 = require("./tabbable");
var FOCUS_TRAP_GUARD_ATTR = 'focusTrapGuard';
function getActiveElement() {
    return document.activeElement;
}
exports.getActiveElement = getActiveElement;
function isElementInScope(scope, element) {
    return scope.contains(element);
}
exports.isElementInScope = isElementInScope;
function isFocusFree() {
    return getActiveElement() === document.body;
}
exports.isFocusFree = isFocusFree;
function createGuard(tabIndex) {
    var element = document.createElement('span');
    element.tabIndex = tabIndex;
    Object.assign(element.style, {
        width: '1px',
        height: 0,
        padding: 0,
        overflow: 'hidden',
        position: 'fixed',
        top: '1px',
        left: '1px',
    });
    element.setAttribute('aria-hidden', 'true');
    element.dataset[FOCUS_TRAP_GUARD_ATTR] = 'true';
    return element;
}
exports.createGuard = createGuard;
function isGuard(node) {
    return Boolean(node.dataset[FOCUS_TRAP_GUARD_ATTR]);
}
exports.isGuard = isGuard;
function removeElementFromDocument(element) {
    if (!element.parentNode) {
        return;
    }
    element.parentNode.removeChild(element);
}
exports.removeElementFromDocument = removeElementFromDocument;
function focusFirstIn(scope) {
    var _a = tslib_1.__read((0, tabbable_1.getTabbables)(scope, function (node) { return !isGuard(node); }), 1), first = _a[0];
    (0, dom_utils_1.focusElement)(first);
}
exports.focusFirstIn = focusFirstIn;
function getNextTabbableIndex(current, count, step) {
    if (current === -1) {
        return 0;
    }
    return count > 0 ? (count + current + step) % count : -1;
}
exports.getNextTabbableIndex = getNextTabbableIndex;
