"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.adjustedTabIndex = void 0;
var ElementTreeWalker_1 = require("./ElementTreeWalker");
var utils_1 = require("./utils");
function getCheckedRadio(radio) {
    if (radio.checked) {
        return radio;
    }
    var walker = new ElementTreeWalker_1.ElementTreeWalker(radio.form || radio.ownerDocument.body, function (element) {
        return ((0, utils_1.isRadioInput)(element) &&
            element.name === radio.name &&
            element.checked &&
            element.form === radio.form);
    });
    return walker.next();
}
function isTabbableRadio(radio) {
    if (!radio.name) {
        return true;
    }
    var checked = getCheckedRadio(radio);
    return !checked || checked === radio;
}
function adjustedTabIndex(node) {
    if (((0, utils_1.isRadioInput)(node) && !isTabbableRadio(node)) || (0, utils_1.isIframe)(node)) {
        return -1;
    }
    var tabIndex = parseInt(node.getAttribute('tabindex') || '', 10);
    if (!isNaN(tabIndex)) {
        return tabIndex;
    }
    // Use fallback value for dom nodes with `contentEditable`,
    // because browsers not returns correct `tabIndex` value.
    if (node.contentEditable === 'true') {
        return 0;
    }
    // Use `tabIndex` fallback value for <details/>, <audio controls/> and <video controls/>,
    // because Chrome returns "-1" and Firefox returns "0" when `tabIndex` not set.
    if ((node.nodeName === 'AUDIO' || node.nodeName === 'VIDEO' || node.nodeName === 'DETAILS') &&
        node.getAttribute('tabindex') === null) {
        return 0;
    }
    return node.tabIndex;
}
exports.adjustedTabIndex = adjustedTabIndex;
