"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFocusManager = void 0;
var tslib_1 = require("tslib");
var dom_utils_1 = require("../dom-utils");
var FocusNavigation_1 = require("./FocusNavigation");
function createFocusManager(scopeRef) {
    var instance = null;
    function ensureNavigation(callback) {
        if (scopeRef.current && (instance === null || instance === void 0 ? void 0 : instance.scope) !== scopeRef.current) {
            instance = new FocusNavigation_1.FocusNavigation(scopeRef.current);
        }
        if (instance) {
            return callback(instance);
        }
        return null;
    }
    function focusFirst(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, tabbable = options.tabbable;
        var element = ensureNavigation(function (navigation) { return navigation.first(tabbable); });
        (0, dom_utils_1.focusElement)(element, preventScroll);
        return element;
    }
    function focusLast(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, tabbable = options.tabbable;
        var element = ensureNavigation(function (navigation) { return navigation.last(tabbable); });
        (0, dom_utils_1.focusElement)(element, preventScroll);
        return element;
    }
    function focusNext(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, opts = tslib_1.__rest(options, ["preventScroll"]);
        var element = ensureNavigation(function (navigation) { return navigation.next(opts); });
        (0, dom_utils_1.focusElement)(element, preventScroll);
        return element;
    }
    function focusPrevious(options) {
        if (options === void 0) { options = {}; }
        var preventScroll = options.preventScroll, opts = tslib_1.__rest(options, ["preventScroll"]);
        var element = ensureNavigation(function (navigation) { return navigation.previous(opts); });
        (0, dom_utils_1.focusElement)(element, preventScroll);
        return element;
    }
    return {
        focusFirst: focusFirst,
        focusLast: focusLast,
        focusNext: focusNext,
        focusPrevious: focusPrevious,
    };
}
exports.createFocusManager = createFocusManager;
