"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useForwardedRef = void 0;
var react_1 = require("react");
/**
 * Copies forwarded ref to mutable ref object.
 *
 * @example
 * const Component = forwardRef<HTMLDivElement>((_, forwardedRef) => {
 *   const ref = useRef<HTMLDivElement>(null)
 *   useForwardedRef(ref, forwardedRef)
 *
 *   return <div ref={ref} />
 * })
 */
function useForwardedRef(ref, forwardedRef) {
    (0, react_1.useImperativeHandle)(forwardedRef, function () { return ref.current; });
}
exports.useForwardedRef = useForwardedRef;
