"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDateFormatter = void 0;
var react_1 = require("react");
var useLocale_1 = require("./useLocale");
var formatterCache = new Map();
function useDateFormatter(options) {
    var locale = (0, useLocale_1.useLocale)().locale;
    var cacheKey = (0, react_1.useMemo)(function () { return getCacheKey(locale, options); }, [locale, options]);
    var cacheValue = formatterCache.get(cacheKey);
    if (cacheValue) {
        return cacheValue;
    }
    var formatter = new Intl.DateTimeFormat(locale, options);
    formatterCache.set(cacheKey, formatter);
    return formatter;
}
exports.useDateFormatter = useDateFormatter;
function getCacheKey(locale, options) {
    var salt = '';
    if (options !== undefined) {
        salt = Object.keys(options)
            .map(function (key) { return [key, options[key]]; })
            .sort(function (a, b) { return (a[0] < b[0] ? -1 : 1); })
            .join();
    }
    return locale + salt;
}
