"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unlock = exports.lock = void 0;
var tslib_1 = require("tslib");
var dom_utils_1 = require("../dom-utils");
var platform_1 = require("../platform");
var StandartScrollLocker = tslib_1.__importStar(require("./StandartScrollLocker"));
var TouchScrollLocker = tslib_1.__importStar(require("./TouchScrollLocker"));
function ensureElement(element) {
    return element || document.body;
}
/**
 * Adds a lock on content scrolling on a DOM-element.
 *
 * @param container Link on DOM-element. Default: `document.body`
 */
function lock(container) {
    if (!dom_utils_1.canUseDom) {
        return;
    }
    var element = ensureElement(container);
    StandartScrollLocker.lock(element);
    if ((0, platform_1.isIOS)()) {
        TouchScrollLocker.lock(element);
    }
}
exports.lock = lock;
/**
 * Removes the lock on content scrolling on a DOM-element.
 *
 * @param container Link on DOM-element. Default: `document.body`
 */
function unlock(container) {
    if (!dom_utils_1.canUseDom) {
        return;
    }
    var element = ensureElement(container);
    StandartScrollLocker.unlock(element);
    if ((0, platform_1.isIOS)()) {
        TouchScrollLocker.unlock(element);
    }
}
exports.unlock = unlock;
