import { Key } from 'react';
export declare type SelectionMode = 'none' | 'single' | 'multiple';
export interface UseSelectionStateProps {
    defaultSelectedKeys?: Key[];
    selectionMode: SelectionMode;
    disallowEmptySelection?: boolean;
    onSelectionChange?: (keys: Set<Key>) => void;
}
export interface UseSelectionStateResult {
    selectedKeys: Set<Key>;
    isSelected: (key: Key) => boolean;
    select: (key: Key) => void;
}
/**
 * Manages state for single and multiple selection.
 *
 * @example
 * const Component = (props) => {
 *   const selectionState = useSelectionState({
 *      selectionMode: 'multiple',
 *      defaultSelectedKeys: ['a', 'b'],
 *      onSelectionChange: (keys) => {
 *         // ...
 *      },
 *   });
 * })
 */
export declare function useSelectionState(props: UseSelectionStateProps): UseSelectionStateResult;
