"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSelectionState = void 0;
var tslib_1 = require("tslib");
var react_1 = require("react");
/**
 * Manages state for single and multiple selection.
 *
 * @example
 * const Component = (props) => {
 *   const selectionState = useSelectionState({
 *      selectionMode: 'multiple',
 *      defaultSelectedKeys: ['a', 'b'],
 *      onSelectionChange: (keys) => {
 *         // ...
 *      },
 *   });
 * })
 */
function useSelectionState(props) {
    var defaultSelectedKeys = props.defaultSelectedKeys, onSelectionChange = props.onSelectionChange, selectionMode = props.selectionMode, _a = props.disallowEmptySelection, disallowEmptySelection = _a === void 0 ? false : _a;
    var _b = tslib_1.__read((0, react_1.useState)(function () { return new Set(defaultSelectedKeys); }), 2), selectedKeys = _b[0], setSelectedKeys = _b[1];
    var onSelectionChangeRef = (0, react_1.useRef)(onSelectionChange);
    var select = (0, react_1.useCallback)(function (key) {
        var _a;
        var isExists = selectedKeys.has(key);
        if (selectionMode === 'none' ||
            (isExists && disallowEmptySelection && selectedKeys.size === 1)) {
            return;
        }
        var keys = new Set(selectedKeys);
        if (isExists) {
            keys.delete(key);
        }
        else {
            if (selectionMode === 'single') {
                keys.clear();
            }
            keys.add(key);
        }
        setSelectedKeys(keys);
        (_a = onSelectionChangeRef.current) === null || _a === void 0 ? void 0 : _a.call(onSelectionChangeRef, keys);
    }, [selectedKeys, selectionMode, disallowEmptySelection]);
    var isSelected = (0, react_1.useCallback)(function (key) { return selectedKeys.has(key); }, [selectedKeys]);
    return {
        selectedKeys: selectedKeys,
        isSelected: isSelected,
        select: select,
    };
}
exports.useSelectionState = useSelectionState;
