"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SlotMap = void 0;
var react_1 = require("react");
var utils_1 = require("./utils");
var SlotMap = /** @class */ (function () {
    function SlotMap(children, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.children = null;
        var defaultSlot = options.defaultSlot;
        var defaultContent = [];
        this.slots = new Map();
        if (defaultSlot && !(0, utils_1.isSlotComponent)(defaultSlot)) {
            throw new Error('Invalid default slot component. Should be a component created using "createSlot".');
        }
        react_1.Children.forEach(children, function (child) {
            if ((0, utils_1.isSlotElement)(child)) {
                var type = child.type, props = child.props;
                if (_this.slots.has(type)) {
                    throw new Error("Duplicate slot elements with name \"".concat(type.__slotName, "\" found."));
                }
                _this.slots.set(type, {
                    name: type.__slotName,
                    props: props,
                    rendered: props.children,
                });
            }
            else if (child !== null && child !== undefined) {
                defaultContent.push(child);
            }
        });
        var defaultSlotItem = defaultSlot ? this.slots.get(defaultSlot) : undefined;
        var hasContent = defaultContent.length > 0;
        if (defaultSlotItem && hasContent) {
            throw new Error("Extraneous children found when component already has explicitly default slot with name \"".concat(defaultSlotItem.name, "\"."));
        }
        if (defaultSlotItem) {
            this.children = defaultSlotItem.rendered;
        }
        else if (hasContent) {
            this.children = defaultContent;
        }
    }
    SlotMap.prototype.get = function (slot) {
        if (!(0, utils_1.isSlotComponent)(slot)) {
            throw new Error('Invalid slot component. Should be a component created using "createSlot".');
        }
        return this.slots.get(slot);
    };
    return SlotMap;
}());
exports.SlotMap = SlotMap;
