"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useUniqId = void 0;
var tslib_1 = require("tslib");
var react_1 = require("react");
var dom_utils_1 = require("../dom-utils");
var ssr_1 = require("../ssr");
/**
 * React-hook for generate uniq identifier.
 *
 * When using ssr in a project, you must wrap the application in
 * `SSRProvider` for id synchronization between the server and the client.
 *
 * @param customId - Custom uniq id
 *
 * @example
 * const id = useUniqId()
 */
function useUniqId(customId) {
    var context = (0, react_1.useContext)(ssr_1.SSRContext);
    var _a = tslib_1.__read((0, react_1.useState)(function () { return "xuniq-".concat(context.id, "-").concat(++context.value); }), 1), id = _a[0];
    console.assert(dom_utils_1.canUseDom || context !== ssr_1.initialContextValue, 'When using ssr in a project, you must wrap the application in <SSRProvider>' +
        ' for id synchronization between the server and the client.');
    return customId || id;
}
exports.useUniqId = useUniqId;
