"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCalendar = void 0;
var i18n_1 = require("../../libs/i18n");
var types_1 = require("./types");
function useCalendar(props, state) {
    var readOnly = props.readOnly, disabled = props.disabled;
    var mode = state.mode, focusedDate = state.focusedDate, focusCalendar = state.focusCalendar, focusDate = state.focusDate, moveDate = state.moveDate;
    var isRTL = (0, i18n_1.useLocale)().isRTL;
    var onFocus = function () {
        focusCalendar(true);
    };
    var onBlur = function (event) {
        var relatedTarget = event.relatedTarget, currentTarget = event.currentTarget;
        if (!currentTarget.contains(relatedTarget)) {
            focusCalendar(false);
        }
    };
    var onKeyDown = function (event) {
        var action = null;
        switch (event.key) {
            case 'ArrowLeft':
                action = isRTL ? types_1.CalendarNavigationAction.NextCell : types_1.CalendarNavigationAction.PrevCell;
                break;
            case 'ArrowRight':
                action = isRTL ? types_1.CalendarNavigationAction.PrevCell : types_1.CalendarNavigationAction.NextCell;
                break;
            case 'ArrowUp':
                action = types_1.CalendarNavigationAction.UpperCell;
                break;
            case 'ArrowDown':
                action = types_1.CalendarNavigationAction.LowerCell;
                break;
            case 'Home':
                action = event.shiftKey
                    ? types_1.CalendarNavigationAction.FirstCell
                    : types_1.CalendarNavigationAction.StartCell;
                break;
            case 'End':
                action = event.shiftKey
                    ? types_1.CalendarNavigationAction.LastCell
                    : types_1.CalendarNavigationAction.EndCell;
                break;
            case 'PageUp':
                action = event.shiftKey
                    ? types_1.CalendarNavigationAction.PrevExtraView
                    : types_1.CalendarNavigationAction.PrevView;
                break;
            case 'PageDown':
                action = event.shiftKey
                    ? types_1.CalendarNavigationAction.NextExtraView
                    : types_1.CalendarNavigationAction.NextView;
                break;
        }
        if (action !== null) {
            event.preventDefault();
            focusDate(moveDate(focusedDate, action));
        }
    };
    var gridProps = {
        role: 'grid',
        'aria-readonly': readOnly,
        'aria-disabled': disabled,
        'aria-multiselectable': mode === 'multiple' || mode === 'range',
        onFocus: onFocus,
        onBlur: onBlur,
        onKeyDown: onKeyDown,
    };
    return {
        gridProps: gridProps,
    };
}
exports.useCalendar = useCalendar;
