"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSingleCalendarState = void 0;
var tslib_1 = require("tslib");
var react_1 = require("react");
var date_1 = require("../../internal/date");
var i18n_1 = require("../../libs/i18n");
var utils_1 = require("../../libs/utils");
var constants_1 = require("./constants");
var types_1 = require("./types");
var utils_2 = require("./utils");
function useSingleCalendarState(props) {
    var propValue = props.value, _a = props.defaultFocusedDate, defaultFocusedDate = _a === void 0 ? propValue : _a, onChange = props.onChange, _b = props.autoFocus, autoFocus = _b === void 0 ? false : _b, _c = props.readOnly, readOnly = _c === void 0 ? false : _c, _d = props.disabled, disabled = _d === void 0 ? false : _d, _e = props.min, propMin = _e === void 0 ? constants_1.MIN_DATE : _e, _f = props.max, propMax = _f === void 0 ? constants_1.MAX_DATE : _f, _g = props.viewsCount, viewsCount = _g === void 0 ? 1 : _g, _h = props.minCalendarView, minCalendarView = _h === void 0 ? 'day' : _h, _j = props.defaultCalendarView, defaultCalendarView = _j === void 0 ? minCalendarView : _j, _k = props.maxCalendarView, maxCalendarView = _k === void 0 ? 'year' : _k;
    var min = (0, react_1.useMemo)(function () { return new Date(propMin); }, [propMin]);
    var max = (0, react_1.useMemo)(function () { return new Date(propMax); }, [propMax]);
    var value = (0, react_1.useMemo)(function () { return (propValue ? new Date(propValue) : undefined); }, [propValue]);
    var firstDayOfWeek = (0, i18n_1.useFirstDayOfWeek)();
    var _l = tslib_1.__read((0, react_1.useState)(function () { return normalizeView(defaultCalendarView); }), 2), activeView = _l[0], setActiveView = _l[1];
    var _m = tslib_1.__read((0, react_1.useState)(autoFocus), 2), isCalendarFocused = _m[0], focusCalendar = _m[1];
    var _o = tslib_1.__read((0, react_1.useState)(function () {
        return normalizeFocusedDate(defaultFocusedDate ? new Date(defaultFocusedDate) : new Date());
    }), 2), focusedDate = _o[0], setFocusedDate = _o[1];
    var _p = tslib_1.__read((0, react_1.useState)(function () { return normalizeBaseDate(activeView, focusedDate); }), 2), baseDate = _p[0], setBaseDate = _p[1];
    function normalizeFocusedDate(date) {
        return (0, utils_2.startOfCell)(minCalendarView, (0, utils_2.clampDate)(date, min, max));
    }
    function normalizeBaseDate(view, date) {
        var maxDate = (0, utils_2.subViews)(view, max, viewsCount - 1);
        var candidate = (0, utils_2.startOfView)(view, (0, utils_2.clampDate)(date, min, min > maxDate ? min : maxDate));
        return (0, utils_2.getViewDate)(view, candidate, date);
    }
    function normalizeView(view) {
        var key = (0, utils_1.clamp)(types_1.CalendarViewKind[view], types_1.CalendarViewKind[minCalendarView], types_1.CalendarViewKind[maxCalendarView]);
        return types_1.CalendarViewKind[key];
    }
    function moveView(view, offset) {
        if (offset === void 0) { offset = 0; }
        var key = types_1.CalendarViewKind[view] + offset;
        var nextView = types_1.CalendarViewKind[key];
        return normalizeView(nextView !== null && nextView !== void 0 ? nextView : view);
    }
    function navigateTo(date) {
        var nextBaseDate = (0, utils_2.getViewDate)(activeView, date, focusedDate);
        if ((0, utils_2.isEqualDate)(baseDate, nextBaseDate)) {
            return;
        }
        setBaseDate(nextBaseDate);
        var nextFocusedDate = normalizeFocusedDate((0, utils_2.nearest)(activeView, focusedDate, nextBaseDate, viewsCount));
        if ((0, utils_2.isEqualDate)(focusedDate, nextFocusedDate)) {
            return;
        }
        setFocusedDate(nextFocusedDate);
    }
    function focusTo(view, date) {
        var nextFocusedDate = (0, utils_2.clampDate)((0, utils_2.startOfCell)(minCalendarView, date), min, max);
        if ((0, utils_2.isEqualDate)(focusedDate, nextFocusedDate)) {
            return;
        }
        setFocusedDate(nextFocusedDate);
        var nextBaseDate = (0, utils_2.getViewDate)(view, (0, utils_2.closestViewDate)(view, baseDate, viewsCount, nextFocusedDate), nextFocusedDate);
        if ((0, utils_2.isEqualDate)(baseDate, nextBaseDate)) {
            return;
        }
        setBaseDate(nextBaseDate);
    }
    function setViewAndFocusedDate(view, date) {
        if (types_1.CalendarViewKind[view] < types_1.CalendarViewKind[minCalendarView] ||
            types_1.CalendarViewKind[view] > types_1.CalendarViewKind[maxCalendarView]) {
            return;
        }
        if (view !== activeView) {
            setActiveView(view);
        }
        var nextFocusedDate = normalizeFocusedDate(date);
        if (!(0, utils_2.isEqualDate)(focusedDate, nextFocusedDate)) {
            setFocusedDate(nextFocusedDate);
        }
        var offset = (0, utils_2.durationInViews)(activeView, baseDate, normalizeBaseDate(activeView, nextFocusedDate));
        var nextBaseDate = (0, utils_2.subViews)(view, normalizeBaseDate(view, nextFocusedDate), offset);
        if (!(0, utils_2.isEqualDate)(baseDate, nextBaseDate)) {
            setBaseDate(nextBaseDate);
        }
    }
    function setView(view) {
        setViewAndFocusedDate(view, focusedDate);
    }
    function focusDate(date) {
        focusTo(activeView, date);
    }
    function selectDate(date) {
        if (readOnly || disabled || !(0, utils_2.isInRange)(activeView, date, (0, utils_2.createRange)(min, max))) {
            return;
        }
        var nextView = moveView(activeView, -1);
        if (nextView !== activeView) {
            setViewAndFocusedDate(nextView, date);
        }
        else {
            focusTo(nextView, date);
            if (!readOnly && nextView === minCalendarView) {
                onChange === null || onChange === void 0 ? void 0 : onChange({ value: date });
            }
        }
    }
    function getCellState(cellValue, viewDate) {
        var today = new Date();
        var sameView = (0, utils_2.isSameView)(activeView, cellValue, viewDate);
        var isDisabled = disabled || !(0, utils_2.isInRange)(activeView, cellValue, (0, utils_2.createRange)(min, max));
        var isFocused = sameView && (0, utils_2.isSameCell)(activeView, cellValue, focusedDate);
        var isSelected = value ? (0, utils_2.isSameCell)(activeView, cellValue, value) : false;
        var isToday = (0, utils_2.isSameCell)(activeView, cellValue, today);
        var cellState = {
            isDisabled: isDisabled,
            isFocused: isFocused,
            isSameView: sameView,
            isSelected: isSelected,
            isToday: isToday,
            isWeekend: activeView === 'day' && (0, date_1.isWeekend)(cellValue),
        };
        return cellState;
    }
    function moveDate(date, action) {
        return (0, utils_2.move)(activeView, date, action, { firstDayOfWeek: firstDayOfWeek });
    }
    function canNavigateTo(date) {
        var minViewDate = (0, utils_2.startOfView)(activeView, min);
        var maxViewDate = new Date(Math.max(minViewDate.getTime(), (0, utils_2.startOfView)(activeView, (0, utils_2.subViews)(activeView, max, viewsCount - 1)).getTime()));
        return (0, utils_2.isInRange)(activeView, date, (0, utils_2.createRange)(minViewDate, maxViewDate));
    }
    var views = (0, react_1.useMemo)(function () {
        return (0, utils_2.getViews)(activeView, baseDate, viewsCount, { firstDayOfWeek: firstDayOfWeek });
    }, [baseDate, viewsCount, activeView, firstDayOfWeek]);
    return {
        mode: 'single',
        value: value,
        isCalendarFocused: isCalendarFocused,
        baseDate: baseDate,
        focusedDate: focusedDate,
        activeView: activeView,
        views: views,
        focusCalendar: focusCalendar,
        focusDate: focusDate,
        selectDate: selectDate,
        getCellState: getCellState,
        moveDate: moveDate,
        setView: setView,
        canNavigateTo: canNavigateTo,
        navigateTo: navigateTo,
        moveView: moveView,
    };
}
exports.useSingleCalendarState = useSingleCalendarState;
