"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getViews = exports.getViewDate = exports.closestViewDate = exports.move = exports.getViewData = exports.clampDate = exports.isSameCell = exports.isInRange = exports.isSameView = exports.nearest = exports.subViews = exports.addViews = exports.durationInViews = exports.startOfCell = exports.endOfView = exports.startOfView = exports.createRange = exports.isEqualDate = void 0;
var date_1 = require("../../../internal/date");
var utils_1 = require("../../../libs/utils");
var day_view_1 = require("./day-view");
var month_view_1 = require("./month-view");
var year_view_1 = require("./year-view");
var date_2 = require("../../../internal/date");
Object.defineProperty(exports, "isEqualDate", { enumerable: true, get: function () { return date_2.isEqual; } });
function createRange(start, end) {
    if (!start || !end) {
        return { start: start !== null && start !== void 0 ? start : end, end: null };
    }
    if (end < start) {
        return { start: end, end: start };
    }
    return { start: start, end: end };
}
exports.createRange = createRange;
function startOfView(view, value) {
    var utils = {
        day: date_1.startOfMonth,
        month: date_1.startOfYear,
        year: year_view_1.startOfYearView,
    };
    return utils[view](value);
}
exports.startOfView = startOfView;
function endOfView(view, date) {
    var utils = {
        day: date_1.endOfMonth,
        month: date_1.endOfYear,
        year: year_view_1.endOfYearView,
    };
    return utils[view](date);
}
exports.endOfView = endOfView;
function startOfCell(view, date) {
    var utils = {
        day: date_1.startOfDay,
        month: date_1.startOfMonth,
        year: date_1.startOfYear,
    };
    return utils[view](date);
}
exports.startOfCell = startOfCell;
function durationInViews(view, start, end) {
    var utils = {
        day: day_view_1.durationInDayViews,
        month: month_view_1.durationInMonthViews,
        year: year_view_1.durationInYearViews,
    };
    return utils[view](start, end);
}
exports.durationInViews = durationInViews;
function addViews(view, date, amount) {
    var utils = {
        day: date_1.addMonths,
        month: date_1.addYears,
        year: year_view_1.addYearViews,
    };
    return utils[view](date, amount);
}
exports.addViews = addViews;
function subViews(view, date, amount) {
    var utils = {
        day: date_1.subMonths,
        month: date_1.subYears,
        year: year_view_1.subYearViews,
    };
    return utils[view](date, amount);
}
exports.subViews = subViews;
function nearest(view, candidate, min, total) {
    var distanceToStart = durationInViews(view, candidate, min);
    var distanceToEnd = distanceToStart + total - 1;
    if (distanceToStart > 0 || distanceToEnd < 0) {
        var amount = Math.abs(distanceToStart) < Math.abs(distanceToEnd) ? distanceToStart : distanceToEnd;
        return addViews(view, candidate, amount);
    }
    return candidate;
}
exports.nearest = nearest;
function isSameView(view, leftDate, rightDate) {
    return durationInViews(view, leftDate, rightDate) === 0;
}
exports.isSameView = isSameView;
function isInRange(view, candidate, range) {
    if (!range.start || !range.end) {
        return false;
    }
    var value = startOfCell(view, candidate);
    return value >= startOfCell(view, range.start) && value <= startOfCell(view, range.end);
}
exports.isInRange = isInRange;
function isSameCell(view, leftDate, rightDate) {
    var utils = {
        day: date_1.isSameDay,
        month: date_1.isSameMonth,
        year: date_1.isSameYear,
    };
    return utils[view](leftDate, rightDate);
}
exports.isSameCell = isSameCell;
function clampDate(date, min, max) {
    var timestamp = (0, utils_1.clamp)(date.getTime(), min.getTime(), max.getTime());
    return (0, date_1.startOfDay)(timestamp);
}
exports.clampDate = clampDate;
function getViewData(view, viewDate, options) {
    var utils = {
        day: day_view_1.getDayViewData,
        month: month_view_1.getMonthViewData,
        year: year_view_1.getYearViewData,
    };
    return utils[view](viewDate, options);
}
exports.getViewData = getViewData;
function move(view, date, action, options) {
    var utils = {
        day: day_view_1.moveInDayView,
        month: month_view_1.moveInMonthView,
        year: year_view_1.moveInYearView,
    };
    return utils[view](date, action, options);
}
exports.move = move;
function closestViewDate(view, start, total, target) {
    var maxIndex = total - 1;
    var duration = durationInViews(view, start, target);
    var offset = Math.min(duration, 0) + Math.max(duration, maxIndex) - maxIndex;
    if (offset !== 0) {
        return addViews(view, start, offset);
    }
    return start;
}
exports.closestViewDate = closestViewDate;
// TODO: rename
function getViewDate(view, baseDate, focusedDate) {
    var day = focusedDate.getDate();
    var month = focusedDate.getMonth();
    var utils = {
        day: function (v) { return v; },
        month: function (v) { return (0, date_1.set)(v, { date: day }); },
        year: function (v) { return (0, date_1.set)(v, { date: day, month: month }); },
    };
    return utils[view](baseDate);
}
exports.getViewDate = getViewDate;
function getViews(view, baseDate, count, options) {
    var result = [];
    for (var index = 0; index < count; index++) {
        var viewDate = addViews(view, baseDate, index);
        var start = startOfView(view, viewDate);
        var end = endOfView(view, viewDate);
        var viewRange = createRange(start, end);
        result.push({
            viewDate: viewDate,
            viewRange: viewRange,
            data: getViewData(view, viewDate, options),
        });
    }
    return result;
}
exports.getViews = getViews;
