"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.moveInDayView = exports.getDayViewData = exports.durationInDayViews = void 0;
var date_1 = require("../../../internal/date");
var types_1 = require("../types");
function durationInDayViews(start, end) {
    return (end.getFullYear() - start.getFullYear()) * 12 + (end.getMonth() - start.getMonth());
}
exports.durationInDayViews = durationInDayViews;
function getDayViewData(viewDate, options) {
    if (options === void 0) { options = {}; }
    var _a = options.firstDayOfWeek, firstDayOfWeek = _a === void 0 ? 0 : _a;
    var data = [];
    var month = viewDate.getMonth();
    var year = viewDate.getFullYear();
    var monthStartsAt = ((0, date_1.startOfMonth)(viewDate).getDay() - firstDayOfWeek) % 7;
    if (monthStartsAt < 0) {
        monthStartsAt += 7;
    }
    var daysInMonth = (0, date_1.getDaysInMonth)(viewDate);
    var weeksInMonth = Math.ceil((monthStartsAt + daysInMonth) / 7);
    for (var weekIndex = 0; weekIndex < weeksInMonth; weekIndex++) {
        data.push([]);
        for (var dayIndex = 0; dayIndex < 7; dayIndex++) {
            var day = weekIndex * 7 + dayIndex - monthStartsAt + 1;
            var value = new Date(year, month, day);
            data[weekIndex].push(value);
        }
    }
    return data;
}
exports.getDayViewData = getDayViewData;
function moveInDayView(date, action, options) {
    if (options === void 0) { options = {}; }
    var firstDayOfWeek = options.firstDayOfWeek;
    switch (action) {
        case types_1.CalendarNavigationAction.NextCell:
            return (0, date_1.addDays)(date, 1);
        case types_1.CalendarNavigationAction.PrevCell:
            return (0, date_1.subDays)(date, 1);
        case types_1.CalendarNavigationAction.LowerCell:
            return (0, date_1.addWeeks)(date, 1);
        case types_1.CalendarNavigationAction.UpperCell:
            return (0, date_1.subWeeks)(date, 1);
        case types_1.CalendarNavigationAction.FirstCell:
            return (0, date_1.startOfMonth)(date);
        case types_1.CalendarNavigationAction.LastCell:
            return (0, date_1.lastDayOfMonth)(date);
        case types_1.CalendarNavigationAction.StartCell:
            return (0, date_1.startOfWeek)(date, { weekStartsOn: firstDayOfWeek });
        case types_1.CalendarNavigationAction.EndCell:
            return (0, date_1.endOfWeek)(date, { weekStartsOn: firstDayOfWeek });
        case types_1.CalendarNavigationAction.PrevView:
            return (0, date_1.subMonths)(date, 1);
        case types_1.CalendarNavigationAction.NextView:
            return (0, date_1.addMonths)(date, 1);
        case types_1.CalendarNavigationAction.PrevExtraView:
            return (0, date_1.subYears)(date, 1);
        case types_1.CalendarNavigationAction.NextExtraView:
            return (0, date_1.addYears)(date, 1);
        default:
            return date;
    }
}
exports.moveInDayView = moveInDayView;
