"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.moveInYearView = exports.getYearViewData = exports.subYearViews = exports.addYearViews = exports.durationInYearViews = exports.endOfYearView = exports.startOfYearView = void 0;
var date_1 = require("../../../internal/date");
var types_1 = require("../types");
var YEAR_VIEW_ROW_COUNT = 6;
var YEAR_VIEW_COL_COUNT = 4;
var YEAR_VIEW_TOTAL_COUNT = YEAR_VIEW_ROW_COUNT * YEAR_VIEW_COL_COUNT;
function getStartYearOfView(date) {
    return Math.floor(date.getFullYear() / YEAR_VIEW_TOTAL_COUNT) * YEAR_VIEW_TOTAL_COUNT;
}
function getEndYearOfView(date) {
    return getStartYearOfView(date) + YEAR_VIEW_TOTAL_COUNT - 1;
}
function startOfYearView(date) {
    var year = getStartYearOfView(date);
    return (0, date_1.startOfYear)((0, date_1.setYear)(date, year));
}
exports.startOfYearView = startOfYearView;
function endOfYearView(date) {
    var year = getEndYearOfView(date);
    return (0, date_1.endOfYear)((0, date_1.setYear)(date, year));
}
exports.endOfYearView = endOfYearView;
function durationInYearViews(start, end) {
    var startYear = getStartYearOfView(start);
    var endYear = getStartYearOfView(end);
    return (endYear - startYear) / YEAR_VIEW_TOTAL_COUNT;
}
exports.durationInYearViews = durationInYearViews;
function addYearViews(date, amount) {
    return (0, date_1.addYears)(date, amount * YEAR_VIEW_TOTAL_COUNT);
}
exports.addYearViews = addYearViews;
function subYearViews(date, amount) {
    return (0, date_1.subYears)(date, amount * YEAR_VIEW_TOTAL_COUNT);
}
exports.subYearViews = subYearViews;
function getYearViewData(viewDate) {
    var data = [];
    for (var row = 0, index = 0; row < YEAR_VIEW_ROW_COUNT; row++) {
        data.push([]);
        for (var col = 0; col < YEAR_VIEW_COL_COUNT; col++) {
            data[row].push((0, date_1.addYears)(viewDate, index++));
        }
    }
    return data;
}
exports.getYearViewData = getYearViewData;
function moveInYearView(date, action) {
    switch (action) {
        case types_1.CalendarNavigationAction.NextCell:
            return (0, date_1.addYears)(date, 1);
        case types_1.CalendarNavigationAction.PrevCell:
            return (0, date_1.subYears)(date, 1);
        case types_1.CalendarNavigationAction.LowerCell:
            return (0, date_1.addYears)(date, YEAR_VIEW_COL_COUNT);
        case types_1.CalendarNavigationAction.UpperCell:
            return (0, date_1.subYears)(date, YEAR_VIEW_COL_COUNT);
        case types_1.CalendarNavigationAction.StartCell:
        case types_1.CalendarNavigationAction.FirstCell:
            return (0, date_1.setYear)(date, getStartYearOfView(date));
        case types_1.CalendarNavigationAction.EndCell:
        case types_1.CalendarNavigationAction.LastCell:
            return (0, date_1.setYear)(date, getEndYearOfView(date));
        case types_1.CalendarNavigationAction.PrevView:
            return subYearViews(date, 1);
        case types_1.CalendarNavigationAction.NextView:
            return addYearViews(date, 1);
        default:
            return date;
    }
}
exports.moveInYearView = moveInYearView;
