"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDateTimeFieldSegment = void 0;
var tslib_1 = require("tslib");
var react_1 = require("react");
var press_1 = require("../../interactions/press");
var focus_1 = require("../../libs/focus");
var i18n_1 = require("../../libs/i18n");
var merge_props_1 = require("../../libs/merge-props");
var spinbutton_1 = require("../spinbutton");
var utils_1 = require("./utils");
function useDateTimeFieldSegment(props, state) {
    var segment = props.segment;
    var type = segment.type, value = segment.value, max = segment.max, text = segment.text, isValid = segment.isValid, isDisabled = segment.isDisabled;
    var resolvedOptions = state.resolvedOptions, increment = state.increment, decrement = state.decrement, extraIncrement = state.extraIncrement, extraDecrement = state.extraDecrement, incrementToMax = state.incrementToMax, decrementToMin = state.decrementToMin, setSegmentValue = state.setSegmentValue;
    var isRTL = (0, i18n_1.useLocale)().isRTL;
    var _a = (0, focus_1.useFocusManager)(), focusNext = _a.focusNext, focusPrevious = _a.focusPrevious;
    var _b = tslib_1.__read((0, react_1.useState)(''), 2), enteredKeys = _b[0], setEnteredKeys = _b[1];
    var ariaValueAttrs = (0, utils_1.getAriaValueAttributes)(segment, resolvedOptions);
    var spinButtonProps = (0, spinbutton_1.useSpinButton)(tslib_1.__assign(tslib_1.__assign({}, ariaValueAttrs), { textValue: text, disabled: isDisabled, onIncrement: (0, react_1.useCallback)(function () { return increment(type); }, [increment, type]), onDecrement: (0, react_1.useCallback)(function () { return decrement(type); }, [decrement, type]), onExtraIncrement: (0, react_1.useCallback)(function () { return extraIncrement(type); }, [extraIncrement, type]), onExtraDecrement: (0, react_1.useCallback)(function () { return extraDecrement(type); }, [extraDecrement, type]), onIncrementToMax: (0, react_1.useCallback)(function () { return incrementToMax(type); }, [incrementToMax, type]), onDecrementToMin: (0, react_1.useCallback)(function () { return decrementToMin(type); }, [decrementToMin, type]) })).spinButtonProps;
    var pressProps = (0, press_1.usePress)({
        onPressStart: function (event) {
            if (event.source === 'mouse') {
                event.currentTarget.focus();
            }
        },
    }).pressProps;
    var setValue = (0, react_1.useCallback)(function (v) {
        setSegmentValue(type, v);
        if (v === null || v * 10 >= max) {
            setEnteredKeys('');
        }
    }, [max, setSegmentValue, type]);
    var onKeyDown = (0, react_1.useCallback)(function (event) {
        // TODO: check the use of special keys for entering digits in different locales (ex. ar-AE)
        if (['Tab', 'Shift', 'Alt', 'Meta'].includes(event.key) || event.ctrlKey || event.metaKey) {
            return;
        }
        event.preventDefault();
        var focusOptions = { from: event.currentTarget, tabbable: true };
        switch (event.key) {
            case 'ArrowLeft': {
                var focusElement = isRTL ? focusNext : focusPrevious;
                focusElement(focusOptions);
                break;
            }
            case 'ArrowRight': {
                var focusElement = isRTL ? focusPrevious : focusNext;
                focusElement(focusOptions);
                break;
            }
            case 'Backspace': {
                if (isDisabled || value === null) {
                    focusPrevious(focusOptions);
                }
                else {
                    setValue(null);
                }
                break;
            }
            default:
                var newValue = enteredKeys + event.key;
                var numberValue = (0, utils_1.parseSegmentValue)(type, newValue);
                // TODO: check readOnly
                if (isDisabled || numberValue === null) {
                    return;
                }
                event.stopPropagation();
                setValue(numberValue);
                var maxLen = max.toString().length;
                // if the number with next digit capacity is greater than the maximum,
                // move the focus to the next segment
                if (newValue.length >= maxLen || numberValue * 10 >= max) {
                    focusNext(focusOptions);
                }
                else {
                    setEnteredKeys(newValue);
                }
        }
    }, [enteredKeys, focusNext, focusPrevious, isDisabled, isRTL, max, setValue, type, value]);
    var segmentProps = {
        // TODO: set aria props
        // 'aria-label': '',
        // 'aria-labelledby': '',
        // TODO: may be move to useDateTimeField?
        'aria-invalid': !isValid,
        tabIndex: isDisabled ? undefined : 0,
        onKeyDown: onKeyDown,
        onFocus: (0, react_1.useCallback)(function () {
            setEnteredKeys('');
        }, []),
    };
    if (!isDisabled) {
        segmentProps.inputMode = 'numeric';
        // TODO: Voice over incorrect work with contentEditable
        segmentProps.contentEditable = true;
        segmentProps.suppressContentEditableWarning = true;
    }
    return {
        segmentProps: (0, merge_props_1.mergeProps)(spinButtonProps, pressProps, segmentProps),
    };
}
exports.useDateTimeFieldSegment = useDateTimeFieldSegment;
