"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDateTimeFieldState = void 0;
var tslib_1 = require("tslib");
var react_1 = require("react");
var i18n_1 = require("../../libs/i18n");
var constants_1 = require("./constants");
var DateComponents_1 = require("./utils/DateComponents");
var DateTimeFieldAdapter_1 = require("./utils/DateTimeFieldAdapter");
function useDateTimeFieldState(props) {
    var _a = props.value, propValue = _a === void 0 ? null : _a, propMin = props.min, propMax = props.max, propPlaceholder = props.placeholder, formatOptions = props.formatOptions, disabled = props.disabled, readOnly = props.readOnly, onChange = props.onChange;
    var lastValueChange = (0, react_1.useRef)(null);
    var formatter = (0, i18n_1.useDateFormatter)(formatOptions);
    var adapter = (0, react_1.useMemo)(function () {
        return new DateTimeFieldAdapter_1.DateTimeFieldAdapter({
            formatter: formatter,
            min: propMin,
            max: propMax,
            placeholder: propPlaceholder,
            disabled: disabled,
        });
    }, [formatter, propMax, propMin, propPlaceholder, disabled]);
    var _b = tslib_1.__read((0, react_1.useState)(function () {
        return adapter.normalizeDateComponents(DateComponents_1.DateComponents.from(propValue));
    }), 2), localValue = _b[0], setLocalValue = _b[1];
    var value = (0, react_1.useMemo)(function () {
        if (lastValueChange.current !== propValue) {
            return DateComponents_1.DateComponents.from(propValue);
        }
        return localValue;
    }, [propValue, localValue]);
    lastValueChange.current = propValue;
    var segments = (0, react_1.useMemo)(function () { return adapter.getSegments(value); }, [adapter, value]);
    var setSegmentValue = (0, react_1.useCallback)(function (type, segmentValue) {
        if (disabled || readOnly) {
            return;
        }
        var newLocalValue = adapter.setSegmentValue(value, type, segmentValue);
        if (DateComponents_1.DateComponents.isEqual(value, newLocalValue)) {
            return;
        }
        setLocalValue(newLocalValue);
        var date = adapter.toDate(newLocalValue);
        if (date !== lastValueChange.current) {
            onChange === null || onChange === void 0 ? void 0 : onChange({ value: date });
        }
        lastValueChange.current = date;
    }, [disabled, readOnly, adapter, value, onChange]);
    var stepHandler = (0, react_1.useCallback)(function (type, step) {
        var segmentValue = adapter.moveByStep(value, type, step);
        setSegmentValue(type, segmentValue);
    }, [adapter, value, setSegmentValue]);
    var increment = (0, react_1.useCallback)(function (type) { return stepHandler(type, 1); }, [stepHandler]);
    var decrement = (0, react_1.useCallback)(function (type) { return stepHandler(type, -1); }, [stepHandler]);
    var extraIncrement = (0, react_1.useCallback)(function (type) { return stepHandler(type, constants_1.EXTRA_STEP[type] || 1); }, [stepHandler]);
    var extraDecrement = (0, react_1.useCallback)(function (type) { return stepHandler(type, -(constants_1.EXTRA_STEP[type] || 1)); }, [stepHandler]);
    var incrementToMax = (0, react_1.useCallback)(function (type) {
        var max = adapter.getLimits(value, type).max;
        setSegmentValue(type, max);
    }, [adapter, setSegmentValue, value]);
    var decrementToMin = (0, react_1.useCallback)(function (type) {
        var min = adapter.getLimits(value, type).min;
        setSegmentValue(type, min);
    }, [adapter, setSegmentValue, value]);
    return {
        resolvedOptions: adapter.resolvedOptions,
        segments: segments,
        setSegmentValue: setSegmentValue,
        increment: increment,
        decrement: decrement,
        extraIncrement: extraIncrement,
        extraDecrement: extraDecrement,
        incrementToMax: incrementToMax,
        decrementToMin: decrementToMin,
    };
}
exports.useDateTimeFieldState = useDateTimeFieldState;
