"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OverlayManager = void 0;
var overlays = [];
var eventTarget = null;
var activeOverlay = null;
function getTopOverlayOptions() {
    return overlays[overlays.length - 1];
}
function isEssentionalClick(refs, target) {
    return refs.some(function (ref) {
        return ref.current instanceof HTMLElement && ref.current.contains(target);
    });
}
function handleClose(event, source, strategy) {
    var options = getTopOverlayOptions();
    if (!options || !options.onClose || (strategy && strategy !== options.closeStrategy)) {
        return;
    }
    if (source === 'click' && isEssentionalClick(options.refs, event.target)) {
        return;
    }
    options.onClose({ nativeEvent: event, source: source });
}
function onDocumentKeyUp(event) {
    var key = event.key;
    if (key !== 'Escape' && key !== 'Esc') {
        return;
    }
    handleClose(event, 'esc');
}
function onDocumentPointerDown(event) {
    eventTarget = event.target;
    activeOverlay = getTopOverlayOptions();
    handleClose(event, 'click', 'pressdown');
}
function onDocumentClick(event) {
    var target = eventTarget;
    eventTarget = null;
    var currentActiveOverlay = activeOverlay;
    activeOverlay = null;
    if (event.button > 0) {
        return;
    }
    // Проверяем, что слой тот же, что при `mousedown`,
    // иначе можем закрыть 2 слоя за одно нажатие
    if (currentActiveOverlay !== getTopOverlayOptions()) {
        return;
    }
    // Убеждаемся, что элемент, который был нажат, совпадает с последним
    // при срабатывании события mousedown. Это предотвращает закрытие диалогового окна
    // перетаскиванием курсора (например, выделением текста внутри диалогового окна
    // и отпусканием мыши за его пределами).
    if (target !== event.target) {
        return;
    }
    handleClose(event, 'click', 'pressup');
}
function attachEvents() {
    document.addEventListener('keyup', onDocumentKeyUp);
    document.addEventListener('pointerdown', onDocumentPointerDown, true);
    document.addEventListener('click', onDocumentClick, true);
}
function detachEvents() {
    document.removeEventListener('keyup', onDocumentKeyUp);
    document.removeEventListener('pointerdown', onDocumentPointerDown, true);
    document.removeEventListener('click', onDocumentClick, true);
}
exports.OverlayManager = Object.freeze({
    count: function () {
        return overlays.length;
    },
    addOverlay: function (options) {
        if (overlays.length === 0) {
            attachEvents();
        }
        overlays.push(options);
    },
    removeOverlay: function (options) {
        overlays.splice(overlays.indexOf(options), 1);
        if (overlays.length === 0) {
            detachEvents();
        }
    },
    getTopOverlayOptions: getTopOverlayOptions,
});
