"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useRadio = void 0;
var focusable_1 = require("../../interactions/focusable");
var press_1 = require("../../interactions/press");
var merge_props_1 = require("../../libs/merge-props");
var platform_1 = require("../../libs/platform");
var useRadioGroupContext_1 = require("./useRadioGroupContext");
function useRadio(props, inputRef) {
    var _a;
    var propChecked = props.checked, defaultChecked = props.defaultChecked, disabled = props.disabled, name = props.name, onChange = props.onChange, propReadOnly = props.readOnly, state = props.state, value = props.value;
    var focusableProps = (0, focusable_1.useFocusable)(props, inputRef).focusableProps;
    var _b = (0, press_1.usePress)(props), isPressed = _b.isPressed, pressProps = _b.pressProps;
    var radioGroupContext = (0, useRadioGroupContext_1.useRadioGroupContext)();
    var checked = radioGroupContext
        ? radioGroupContext.selectedValue === props.value
        : propChecked !== null && propChecked !== void 0 ? propChecked : defaultChecked;
    var readOnly = (radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.isReadOnly) || propReadOnly;
    if (radioGroupContext) {
        if (process.env.NODE_ENV !== 'production') {
            if (props.checked || props.defaultChecked) {
                console.warn('Using checked/defaultChecked prop with RadioGroupContext will have no effect');
            }
            if (props.onChange) {
                console.warn('Using onChange prop with RadioGroupContext will have no effect');
            }
            if (props.name) {
                console.warn('Using name prop with RadioGroupContext will have no effect');
            }
        }
    }
    var handleOnChange = function (event) {
        var _a;
        if (radioGroupContext) {
            (_a = radioGroupContext.setSelectedValue) === null || _a === void 0 ? void 0 : _a.call(radioGroupContext, event);
        }
        else {
            onChange === null || onChange === void 0 ? void 0 : onChange(event);
        }
    };
    return {
        isPressed: isPressed,
        rootProps: pressProps,
        inputProps: (0, merge_props_1.mergeProps)(focusableProps, {
            type: 'radio',
            onChange: readOnly ? undefined : handleOnChange,
            'aria-invalid': state === 'invalid' || undefined,
            'aria-checked': checked || props.defaultChecked,
            // Use "aria-readonly" because "readOnly" available only for text fields,
            // see https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly.
            'aria-readonly': readOnly || undefined,
            // https://bugzilla.mozilla.org/show_bug.cgi?id=654072
            autoComplete: (0, platform_1.isFirefox)() ? 'off' : undefined,
            checked: checked,
            name: (_a = radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.name) !== null && _a !== void 0 ? _a : name,
            disabled: (radioGroupContext === null || radioGroupContext === void 0 ? void 0 : radioGroupContext.isDisabled) || disabled,
            value: value,
        }),
    };
}
exports.useRadio = useRadio;
