"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAutoResize = void 0;
var isomorphic_layout_effect_1 = require("../../libs/isomorphic-layout-effect");
var utils_1 = require("../../libs/utils");
/**
 * Enable auto resize textarea when text changed.
 *
 * @param props - Props for the auto resize.
 * @param inputRef - Ref to the textarea html element.
 */
function useAutoResize(props, inputRef) {
    var value = props.value, enabled = props.enabled;
    var minRows = Math.max(props.minRows || 1, 1);
    var maxRows = (0, utils_1.clamp)(props.maxRows || Infinity, minRows, Infinity);
    (0, isomorphic_layout_effect_1.useIsomorphicLayoutEffect)(function () {
        var input = inputRef.current;
        if (input) {
            input.rows = minRows;
            if (!enabled) {
                return;
            }
            for (var rows = minRows + 1; rows <= maxRows; rows++) {
                if (input.scrollHeight <= input.clientHeight) {
                    break;
                }
                input.rows = rows;
            }
        }
    }, [inputRef, value, enabled]);
}
exports.useAutoResize = useAutoResize;
